@@ -19,6 +19,9 @@
 
 #include <config.h>
 #include "edit.h"
+#if defined (HAVE_MAD) && ! defined (MIDNIGHT) && ! defined (GTK)
+#include "mad.h"
+#endif
 
 /* this file definatively relies on int being 32 bits or more */
 
@@ -51,26 +54,28 @@ extern unsigned long edit_cursor_color;
 extern int EditExposeRedraw;
 extern int EditClear;
 
-void set_style_color (
+int set_style_color (
 #ifdef GTK
 			 Window win,
 #endif
 		      cache_type s, unsigned long *fg, unsigned long *bg)
 {
-    int fgp, bgp;
+    int fgp, bgp, underlined = 0;
     fgp = (s & 0xFF000000UL) >> 24;
 /* NO_COLOR would give fgp == 255 */
-    if (fgp < 255)
+    if (fgp < 0xFF)
 	*fg = color_palette (fgp);
     else
 	*fg = edit_normal_foreground_color;
     bgp = (s & 0x00FF0000) >> 16;
-    if (bgp < 255)
+    if (bgp == 0xFE)
+	underlined = 1;
+    if (bgp < 0xFD)
 	*bg = color_palette (bgp);
     else
 	*bg = edit_normal_background_color;
     if (!(s & 0xFFFFFF00UL))	/* check this first as an optimization */
-	return;
+	return underlined;
     if (s & (MOD_ABNORMAL * 256)) {
 	*bg = edit_abnormal_color;
 	if (s & (MOD_MARKED * 256))
@@ -92,6 +97,7 @@ void set_style_color (
 	if (*bg == COLOR_BLACK)
 	    *bg = color_palette (1);
     }
+    return underlined;
 }
 
 #ifdef GTK
@@ -105,30 +111,13 @@ static inline int next_tab_pos (int x)
     return x += tab_width - x % tab_width;
 }
 
-/* For Ryan: */
-/* converts a possibly unprintable character to a string,
-   returning the string's width in pixels, t must have space for 4 chars */
+/* this now properly uses ctypes */
 static inline int convert_to_long_printable (int c, unsigned char *t)
 {
-    if (c > ' ') {
-	if (c <= '~') {
-	    t[0] = c;
-	    t[1] = 0;
-	    return per_char[c];
-	}
-	if (c >= 160)
-	    if (option_international_characters) {
-		t[0] = c;
-		t[1] = 0;
-		return per_char[c];
-	    }
-	if (c > '~') {
-	    t[0] = ("0123456789ABCDEF")[c >> 4];
-	    t[1] = ("0123456789ABCDEF")[c & 0xF];
-	    t[2] = 'h';
-	    t[3] = 0;
-	    return per_char[t[0]] + per_char[t[1]] + per_char[t[2]];
-	}
+    if (isgraph (c)) {
+	t[0] = c;
+	t[1] = 0;
+	return per_char[c];
     }
     if (c == ' ') {
 	if (option_long_whitespace) {
@@ -142,6 +131,18 @@ static inline int convert_to_long_printable (int c, unsigned char *t)
 	    return per_char[' '];
 	}
     }
+    if (option_international_characters && per_char[c]) {
+	t[0] = c;
+	t[1] = 0;
+	return per_char[c];
+    }
+    if (c > '~') {
+	t[0] = ("0123456789ABCDEF")[c >> 4];
+	t[1] = ("0123456789ABCDEF")[c & 0xF];
+	t[2] = 'h';
+	t[3] = 0;
+	return per_char[t[0]] + per_char[t[1]] + per_char[t[2]];
+    }
     t[0] = '^';
     t[1] = c + '@';
     t[2] = 0;
@@ -151,21 +152,18 @@ static inline int convert_to_long_printable (int c, unsigned char *t)
 /* same as above but just gets the length */
 static inline int width_of_long_printable (int c)
 {
-    if (c > ' ') {
-	if (c <= '~')
-	    return per_char[c];
-	if (c >= 160)
-	    if (option_international_characters)
-		return per_char[c];
-	if (c > '~')
-	    return per_char[(unsigned char) ("0123456789ABCDEF")[c >> 4]] + per_char[(unsigned char) ("0123456789ABCDEF")[c & 0xF]] + per_char['h'];
-    }
+    if (isgraph (c))
+	return per_char[c];
     if (c == ' ') {
 	if (option_long_whitespace)
 	    return per_char[' '] + per_char[' '];
 	else
 	    return per_char[' '];
     }
+    if (option_international_characters && per_char[c])
+	return per_char[c];
+    if (c > '~')
+	return per_char[(unsigned char) ("0123456789ABCDEF")[c >> 4]] + per_char[(unsigned char) ("0123456789ABCDEF")[c & 0xF]] + per_char[(unsigned char) 'h'];
     return per_char['^'] + per_char[c + '@'];
 }
 
@@ -249,6 +247,18 @@ long edit_move_forward3 (WEdit * edit, long current, int pixels, long upto)
 
 extern int column_highlighting;
 
+/* gets the characters style (eg marked, highlighted) from its position in the edit buffer */
+static inline cache_type get_style_fast (WEdit * edit, long q, int c)
+{
+    cache_type s = 0;
+    unsigned int fg, bg;
+    if (!(isprint (c) || (option_international_characters && per_char[c])))
+	if (c != '\n' && c != '\t')
+	    s |= MOD_ABNORMAL * 256;
+    edit_get_syntax_color (edit, q, (int *) &fg, (int *) &bg);
+    return s | ((fg & 0xFF) << 24) | ((bg & 0xFF) << 16);
+}
+
 /* gets the characters style (eg marked, highlighted) from its position in the edit buffer */
 static inline cache_type get_style (WEdit * edit, long q, int c, long m1, long m2, int x)
 {
@@ -269,13 +279,9 @@ static inline cache_type get_style (WEdit * edit, long q, int c, long m1, long m
 	s |= MOD_BOLD * 256;
     if (q >= edit->found_start && q < edit->found_start + edit->found_len)
 	s |= MOD_HIGHLIGHTED * 256;
-    if (option_international_characters) {
-	if ((c < ' ' || (c > '~' && c < 160)) && c != '\t' && c != '\n')
+    if (!(isprint (c) || (option_international_characters && per_char[c])))
+	if (c != '\n' && c != '\t')
 	    s |= MOD_ABNORMAL * 256;
-    } else {
-	if ((c < ' ' || c > '~') && c != '\t' && c != '\n')
-	    s |= MOD_ABNORMAL * 256;
-    }
     edit_get_syntax_color (edit, q, (int *) &fg, (int *) &bg);
     return s | ((fg & 0xFF) << 24) | ((bg & 0xFF) << 16);
 }
@@ -284,72 +290,151 @@ void convert_text (WEdit * edit, long q, cache_type * p, int x, int x_max, int r
 {
     int c;
     cache_type s;
-    long m1, m2;
+    long m1, m2, last;
     unsigned char *r, text[4];
-    int book_mark_colors[10], book_mark, book_mark_cycle = 0, the_end = 0;
-
+    int book_mark_colors[10], book_mark;
     eval_marks (edit, &m1, &m2);
-
     book_mark = book_mark_query_all (edit, edit->start_line + row, book_mark_colors);
-
-    for (;;) {
-	c = edit_get_byte (edit, q);
-	if (!the_end)
-	    *p = get_style (edit, q, c, m1, m2, x);
-	if (book_mark) {
+    last = q + (x_max - x) / 2 + 2;	/* for optimization, we say that the last character 
+					   of this line cannot have an offset greater than this.
+					   This can be used to rule out uncommon text styles,
+					   like a character with a cursor, or selected text */
+    if (book_mark) {
+	int the_end = 0, book_mark_cycle = 0;
+	for (;;) {
+	    c = edit_get_byte (edit, q);
+	    if (!the_end)
+		*p = get_style (edit, q, c, m1, m2, x);
 	    if (the_end)
 		*p = 0;
 	    *p = (*p & 0x0000FFFF) | (book_mark_colors[book_mark_cycle++ % book_mark] << 16);
-	}
-	switch (c) {
-	case '\n':
-	    if (book_mark) {	/* bookmarks must show right across the screen */
+	    switch (c) {
+	    case '\n':
 		the_end = 1;
 		c = ' ';
 		q--;
 		goto the_default;
-	    } else {
+	    case '\t':
+		if (fixed_font) {
+		    int t;
+		    t = next_tab_pos (x);
+		    t = min (t, x_max);
+		    s = *p;
+		    while (x < t) {
+			x += per_char[' '];
+			*p++ = s | ' ';
+		    }
+		} else {
+		    *p++ |= '\t';
+		    x = next_tab_pos (x);
+		}
+		break;
+	    default:
+	      the_default:
+		x += convert_to_long_printable (c, text);
+		r = text;
+		s = *p;
+		*p++ = s | *r++;
+		if (!*r)
+		    break;
+		*p++ = s | *r++;
+		if (!*r)
+		    break;
+		*p++ = s | *r++;
+		break;
+	    }
+	    if (x >= x_max)
+		break;
+	    q++;
+	}
+    } else if ((m2 < q || m1 > last) && (edit->curs1 < q || edit->curs1 > last) && \
+	       (edit->found_start + edit->found_len < q || edit->found_start > last) &&
+	       (edit->bracket < q || edit->bracket > last)) {
+	for (;;) {
+	    c = edit_get_byte (edit, q);
+	    *p = get_style_fast (edit, q, c);
+	    switch (c) {
+	    case '\n':
 		*p++ |= ' ';
 		*p = 0;
 		if (x > edit->max_column)
 		    edit->max_column = x;
 		return;
-	    }
-	case '\t':
-	    if (fixed_font) {
-		int t;
-		t = next_tab_pos (x);
-		t = min (t, x_max);
-		s = *p;
-		while (x < t) {
-#if 0
-		    if (book_mark)
-			*p = (*p & 0x0000FFFF) | (book_mark_colors[book_mark_cycle++ % book_mark] << 16);
-#endif
-		    x += per_char[' '];
-		    *p++ = s | ' ';
+	    case '\t':
+		if (fixed_font) {
+		    int t;
+		    t = next_tab_pos (x);
+		    t = min (t, x_max);
+		    s = *p;
+		    while (x < t) {
+			x += per_char[' '];
+			*p++ = s | ' ';
+		    }
+		} else {
+		    *p++ |= '\t';
+		    x = next_tab_pos (x);
 		}
-	    } else {
-		*p++ |= '\t';
-		x = next_tab_pos (x);
-	    }
-	    break;
-	default:
-	  the_default:
-	    x += convert_to_long_printable (c, text);
-	    r = text;
-	    s = *p;
-	    *p++ = s | *r++;
-	    if (*r) {
+		break;
+	    default:
+		x += convert_to_long_printable (c, text);
+		r = text;
+		s = *p;
 		*p++ = s | *r++;
-		if (*r)
-		    *p++ = s | *r++;
+		if (!*r)
+		    break;
+		*p++ = s | *r++;
+		if (!*r)
+		    break;
+		*p++ = s | *r++;
+		break;
 	    }
-	    break;
+	    if (x >= x_max)
+		break;
+	    q++;
+	}
+    } else {
+	for (;;) {
+	    c = edit_get_byte (edit, q);
+	    *p = get_style (edit, q, c, m1, m2, x);
+	    switch (c) {
+	    case '\n':
+		*p++ |= ' ';
+		*p = 0;
+		if (x > edit->max_column)
+		    edit->max_column = x;
+		return;
+	    case '\t':
+		if (fixed_font) {
+		    int t;
+		    t = next_tab_pos (x);
+		    t = min (t, x_max);
+		    s = *p;
+		    while (x < t) {
+			x += per_char[' '];
+			*p++ = s | ' ';
+		    }
+		} else {
+		    *p++ |= '\t';
+		    x = next_tab_pos (x);
+		}
+		break;
+	    default:
+		x += convert_to_long_printable (c, text);
+		r = text;
+		s = *p;
+		*p++ = s | *r++;
+		if (!*r)
+		    break;
+		*p++ = s | *r++;
+		if (!*r)
+		    break;
+		*p++ = s | *r++;
+		break;
+	    }
+	    if (x >= x_max)
+		break;
+	    q++;
 	}
-	if (x >= x_max)
-	    break;
-	q++;
     }
     if (x > edit->max_column)
 	edit->max_column = x;
@@ -488,6 +573,7 @@ int draw_string (Window win, int x, int y, cache_type s, unsigned char *text, in
     GdkColor fg, bg;
 #else
     unsigned long fg, bg;
+    int underlined, l;
 #endif
 #ifdef GTK
     set_style_color (s, &fg.pixel, &bg.pixel);
@@ -495,12 +581,24 @@ int draw_string (Window win, int x, int y, cache_type s, unsigned char *text, in
     gdk_gc_set_foreground (win->gc, &fg);
     gdk_draw_image_text (win->text_area, GTK_WIDGET (win)->style->font, win->gc, x + FONT_OFFSET_X, y + FONT_OFFSET_Y, text, length);
 #else
-    set_style_color (s, &fg, &bg);
+    underlined = set_style_color (s, &fg, &bg);
     CSetBackgroundColor (bg);
     CSetColor (fg);
     CImageString (win, x + FONT_OFFSET_X, y + FONT_OFFSET_Y, (char *) text, length);
+    l = CTextWidth (win, (char *) text, length);
+    if (underlined) {
+	int i, h, inc;
+	inc = FONT_MEAN_WIDTH * 2 / 3;
+	CSetColor (color_palette (18));
+	h = (x / inc) & 1;
+	CLine (win, x, y + FONT_HEIGHT + FONT_OVERHEAD - 1 - h, x + min (l, inc - (x % inc) - 1), y + FONT_HEIGHT + FONT_OVERHEAD - 1 - h);
+	h = h ^ 1;
+	for (i = inc - min (l, (x % inc)); i < l; i += inc) {
+	    CLine (win, x + i, y + FONT_HEIGHT + FONT_OVERHEAD - 1 - h, x + min (l, i + inc - 1), y + FONT_HEIGHT + FONT_OVERHEAD - 1 - h);
+	    h = h ^ 1;
+	}
+    }
 #endif
-
 /* if we printed a cursor: */
 #ifdef GTK
     if (s & (MOD_CURSOR * 256))
@@ -509,19 +607,18 @@ int draw_string (Window win, int x, int y, cache_type s, unsigned char *text, in
 #else
     if (s & (MOD_CURSOR * 256))
 	edit_set_cursor (win, x, y, bg, fg, per_char[*text], *text);
-    return x + CTextWidth (win, (char *) text, length);
+    return x + l;
 #endif
 }
 
-#define STYLE_DIFF (cache[i] != line[i] \
-	    || ((cache[i] | line[i]) & (MOD_CURSOR * 256)) \
-	    || !cache[i] || !line[i] \
-	    )
+#define STYLE_DIFF (*cache != *line \
+	    || ((*cache | *line) & (MOD_CURSOR * 256)) \
+	    || !*cache || !*line)
 
 int get_ignore_length (cache_type *cache, cache_type *line)
 {
     int i;
-    for (i = 0; i < CACHE_WIDTH; i++) {
+    for (i = 0; i < CACHE_WIDTH; i++, line++, cache++) {
 	if (STYLE_DIFF)
 	    return i;
     }
@@ -554,7 +651,7 @@ int get_ignore_trailer (cache_type *cache, cache_type *line, int length)
 	    if (line[i] != ' ')
 		return i + 1;
 
-    for (i = cache_len - 1; i > length; i--)
+    for (i = cache_len - 1, line = line + i, cache = cache + i; i > length; i--, line--, cache--)
 	if (STYLE_DIFF)
 	    return i + 1;
 
@@ -562,19 +659,32 @@ int get_ignore_trailer (cache_type *cache, cache_type *line, int length)
 }
 
 /* erases trailing bit of old line if a new line is printed over a longer old line */
-void cover_trail (Window win, int x_new, int x_old, int y)
+static void cover_trail (Window win, int x_start, int x_new, int x_old, int y)
 {
     if (x_new < EDIT_TEXT_HORIZONTAL_OFFSET)
-        x_new = EDIT_TEXT_HORIZONTAL_OFFSET;
-    if (x_new >= x_old)		/* no need to print */
-	return;
+	x_new = EDIT_TEXT_HORIZONTAL_OFFSET;
+    if (x_new < x_old) {	/* no need to print */
 #ifdef GTK
-    gdk_gc_set_foreground (win->gc, &win->color[1]);
-    gdk_draw_rectangle (win->text_area, win->gc, 1, x_new, y + FONT_OVERHEAD, x_old - x_new, FONT_HEIGHT);
+	gdk_gc_set_foreground (win->gc, &win->color[1]);
+	gdk_draw_rectangle (win->text_area, win->gc, 1, x_new, y + FONT_OVERHEAD, x_old - x_new, FONT_HEIGHT);
 #else
-    CSetColor (edit_normal_background_color);
-/*    CSetColor (color_palette(12)); */
-    CRectangle (win, x_new, y + FONT_OVERHEAD, x_old - x_new, FONT_HEIGHT);
+	CSetColor (edit_normal_background_color);
+	CRectangle (win, x_new, y + FONT_OVERHEAD, x_old - x_new, FONT_HEIGHT + (FONT_OVERHEAD != 0 && !fixed_font));
+#endif
+    } else {
+#ifdef GTK
+	gdk_gc_set_foreground (win->gc, &win->color[1]);
+#else
+	CSetColor (edit_normal_background_color);
+#endif
+    }
+/* true type fonts print stuff out of the bounding box (aaaaaaaaarrrgh!!) */
+    if (!fixed_font)
+	if (FONT_OVERHEAD && x_new > EDIT_TEXT_HORIZONTAL_OFFSET)
+#ifdef GTK
+	    gdk_draw_line (win->text_area, win->gc, max (x_start, EDIT_TEXT_HORIZONTAL_OFFSET), y + FONT_HEIGHT + FONT_OVERHEAD, x_new - 1, y + FONT_HEIGHT + FONT_OVERHEAD);
+#else
+	    CLine (win, max (x_start, EDIT_TEXT_HORIZONTAL_OFFSET), y + FONT_HEIGHT + FONT_OVERHEAD, x_new - 1, y + FONT_HEIGHT + FONT_OVERHEAD);
 #endif
 }
 
@@ -688,7 +798,7 @@ void edit_draw_proportional (void *data,
 
     x = min (x, x_max);
     if (!EditExposeRedraw || EditClear)
-	cover_trail (win, x, lines[row].x1, y);
+	cover_trail (win, x0, x, lines[row].x1, y);
     memcpy (&(lines[row].data[ignore_text]),
 	    &(line[ignore_text]),
 	 (min (j, CACHE_WIDTH) - ignore_text) * sizeof (cache_type));
@@ -711,7 +821,7 @@ void edit_draw_this_line_proportional (WEdit * edit, long b, int row, int start_
 	return;
 
     if (row + edit->start_line > edit->total_lines)
-	b = 2000000000;		/* force b out of range of the edit buffer for blanks lines */
+	b = edit->last_byte + 1;		/* force b out of range of the edit buffer for blanks lines */
 
     if (end_column > CWidthOf (edit->widget))
 	end_column = CWidthOf (edit->widget);
@@ -733,7 +843,7 @@ void edit_draw_this_line_proportional (WEdit * edit, long b, int row, int start_
 
 static inline int nroff_printable (int c)
 {
-    return ((c >= ' ' && c <= '~') || c >= 160);
+    return isprint (c);
 }
 
 
