@@ -715,6 +715,11 @@ void quick_chdir_cmd (void)
 }
 
 #ifdef USE_VFS
+void free_vfs_now (void)
+{
+    vfs_expire (1);
+}
+
 void reselect_vfs (void)
 {
     char *target;
