@@ -144,7 +144,7 @@
 #include "gsession.h"
 #endif
 
-/* "$Id: main.c,v 1.102 1999/09/29 20:14:44 unammx Exp $" */
+/* "$Id: main.c,v 1.104 1999/11/11 14:35:40 pavel Exp $" */
 
 /* When the modes are active, left_panel, right_panel and tree_panel */
 /* Point to a proper data structure.  You should check with the functions */
@@ -446,7 +446,10 @@ update_one_panel_widget (WPanel *panel, int force_update, char *current_file)
     if (force_update & UP_RELOAD){
 	panel->is_panelized = 0;
 
+#if 1
 	ftpfs_flushdir ();
+#endif
+#warning Should supply flushdir method
 	memset (&(panel->dir_stat), 0, sizeof (panel->dir_stat));
     }
     
@@ -1328,6 +1331,7 @@ static menu_entry CmdMenu [] = {
     { ' ', N_("di&Rectory hotlist    C-\\"),    'R', quick_chdir_cmd },
 #ifdef USE_VFS
     { ' ', N_("&Active VFS list      C-x a"),   'A', reselect_vfs },
+    { ' ', N_("Fr&ee VFSs now"),                'E', free_vfs_now },
 #endif
 #ifdef WITH_BACKGROUND
     { ' ', N_("&Background jobs      C-x j"),   'B', jobs_cmd },
@@ -2373,7 +2377,9 @@ OS_Setup ()
 	
     termvalue = getenv ("TERM");
     if (!termvalue){
+#ifndef HAVE_X
 	fprintf (stderr, _("The TERM environment variable is unset!\n"));
+#endif
 	termvalue = "";
     }
 #ifndef HAVE_X
