@@ -121,7 +121,7 @@ int altered_magic_flag = 0;
 int altered_nroff_flag = 0;
 /* }}} */
 
-/* "$Id: view.c,v 1.34 1999/05/03 18:57:55 unammx Exp $" */
+/* "$Id: view.c,v 1.37 1999/06/24 21:53:49 norbert Exp $" */
 
 static char hex_char[] = "0123456789ABCDEF";
 
@@ -564,15 +564,23 @@ do_view_init (WView *view, char *_command, char *_file, int start_line)
         int fd;
 
 	if ((fd = mc_open(_file, O_RDONLY)) == -1) {
-	    message (1, MSG_ERROR, _(" Can't open \"%s\"\n %s "),
+            char tmp[BUF_MEDIUM];
+	    g_snprintf (tmp, sizeof (tmp), 
+                        _(" Can't open \"%s\"\n %s "),
 			_file, unix_error_string (errno));
-	    return -1;
+	    view->filename = g_strdup (_file);
+            error = set_view_init_error (view, tmp);
+	    goto finish;
 	}
 	if (mc_fstat (fd, &view->s) == -1) {
-	    message (1, MSG_ERROR, _(" Can't open \"%s\"\n %s "),
+            char tmp[BUF_MEDIUM];
+	    g_snprintf (tmp, sizeof (tmp), 
+                        _(" Can't stat \"%s\"\n %s "),
 			_file, unix_error_string (errno));
 	    mc_close(fd);
-	    return -1;
+	    view->filename = g_strdup (_file);
+            error = set_view_init_error (view, tmp);
+	    goto finish;
 	}
 
 	if (_file[0] && view->viewer_magic_flag && (is_gunzipable (fd, &type)) != 0)
@@ -587,6 +595,7 @@ do_view_init (WView *view, char *_command, char *_file, int start_line)
     else
 	error = load_view_file (view, view->filename);
 
+finish:
     if (error){
 	if (!view->have_frame){
 	    message (1, MSG_ERROR, error);
@@ -608,6 +617,7 @@ do_view_init (WView *view, char *_command, char *_file, int start_line)
     /* Special case: The data points to the error message */
     if (error){
 	view->data = error;
+        view->file = -1;
 	view->s.st_size = view->bytes_read = strlen (view->data);
     }
     view->last_byte = view->first + view->s.st_size;
@@ -640,7 +650,7 @@ view_update_bytes_per_line(WView *view)
 	cols = view->widget.cols;
     
     view->bottom_first = -1;
-    view->bytes_per_line = 2 * (cols - 7) / 9;
+    view->bytes_per_line = 2 * (cols - 8) / 9;
     view->bytes_per_line &= 0xfffc;
     
     if (view->bytes_per_line == 0)
@@ -863,15 +873,17 @@ display (WView *view)
 	
 	/* Start of text column */
         int text_start = width - view->bytes_per_line - 1 + frame_shift;
-	
+
         for (;row < height && from < view->last_byte; row++){
             /* Print the hex offset */
-            g_snprintf (hex_buff, sizeof (hex_buff), "%05X", (int) (from - view->first));
-	    widget_move (view, row, frame_shift);
+            view_set_color (view, BOLD_COLOR);
+            g_snprintf (hex_buff, sizeof (hex_buff), "%08X", (int) (from - view->first));
+	    view_gotoyx (view, row, frame_shift);
             view_add_string (view, hex_buff);
+            view_set_color (view, DEF_COLOR);
 	    
-            /* Hex dump starts from column seven */
-            col = 7;
+            /* Hex dump starts from column nine */
+            col = 9;
 	    
             /* Each hex number is two digits */
             hex_buff[2] = 0;
@@ -919,18 +931,22 @@ display (WView *view)
                     /* Hex numbers are printed in the groups of four */
                     /* Groups are separated by a vline */
 
+                    view_gotoyx (view, row, col-1);
                     view_add_character (view, ' ');
+                    view_gotoyx (view, row, col);
                     view_add_one_vline ();
-		    view_gotoyx (view, row, col + 1);
                     col += 2;
                     
                     if (boldflag && from==view->search_start+view->found_len-1)
                     	view_set_color (view, BOLD_COLOR);
 		    
                 }
-                if (boldflag && from < view->search_start + view->found_len - 1 
-                    && bytes != view->bytes_per_line - 1)
-                    view_add_character (view, ' ');
+                if (boldflag && 
+                    from < view->search_start + view->found_len - 1 &&
+                    bytes != view->bytes_per_line - 1) {
+		        view_gotoyx (view, row, col);
+                        view_add_character (view, ' ');
+                }
 		
                 /* Print the corresponding ascii character */
 		view_gotoyx (view, row, text_start + bytes);
@@ -1474,7 +1490,8 @@ cancel_pressed (GtkWidget *widget, int *abort)
 static void
 search (WView *view, char *text, int (*search)(WView *, char *, char *, int))
 {
-    int w = view->widget.cols - (view->have_frame * 2);
+    int w = view->widget.cols - view->have_frame + 1;
+
     char *s = NULL;		/*  The line we read from the view buffer */
     long p, beginning;
     int  ch;
@@ -1615,7 +1632,8 @@ search (WView *view, char *text, int (*search)(WView *, char *, char *, int))
 static long
 block_search (WView *view, char *buffer, int len)
 {
-    int w = view->widget.cols - (view->have_frame * 2);
+    int w = view->widget.cols - view->have_frame + 1;
+
     char *d = buffer, b;
     unsigned long e;
 
@@ -1877,7 +1895,7 @@ toggle_hex_mode (WView *view)
     view_update (view, TRUE);
 }
 
-/* Both views */
+/* Ascii view */
 void
 goto_line (WView *view)
 {
@@ -1905,6 +1923,31 @@ goto_line (WView *view)
     view_update (view, TRUE);
 }
 
+/* Hex view */
+void
+goto_addr (WView *view)
+{
+    char *line, *error, prompt [BUF_SMALL];
+    unsigned long addr;
+
+    g_snprintf (prompt, sizeof (prompt), _(" The current address is 0x%lx.\n"
+		       " Enter the new address:"), view->edit_cursor);
+    line = input_dialog (_(" Goto Address "), prompt, "");
+    if (line){
+	if (*line) {
+	    addr = strtol (line, &error, 0);
+  	    if ((*error == '\0') && (addr <= view->last_byte)) { 
+   	         move_to_top (view);
+		 view_move_forward (view, addr/view->bytes_per_line);
+		 view->edit_cursor = addr;
+            }
+        }
+	g_free (line);
+    }
+    view->dirty++;
+    view_update (view, TRUE);
+}
+
 /* Both views */
 static void
 regexp_search (WView *view, int direction)
@@ -2026,7 +2069,9 @@ view_labels (WView *view)
     
     my_define (h, 10, _("Quit"), view_quit_cmd, view);
     my_define (h, 4, view->hex_mode ? _("Ascii"): _("Hex"), toggle_hex_mode, view);
-    my_define (h, 5, _("Line"), goto_line, view);
+    my_define (h, 5, view->hex_mode ? _("Goto") : _("Line"),
+	        view->hex_mode ? goto_addr : goto_line, 
+		     view);
     my_define (h, 6, view->hex_mode ? _("Save") : _("RxSrch"), regexp_search_cmd, view);
 
     my_define (h, 2, view->hex_mode ? view->hexedit_mode ?
