@@ -23,7 +23,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
  */
-/* "$Id: widget.c,v 1.33 1999/08/06 19:24:05 norbert Exp $" */
+/* "$Id: widget.c,v 1.34 1999/09/14 18:20:03 norbert Exp $" */
 
 #include <config.h>
 #include <string.h>
@@ -1730,7 +1730,7 @@ input_event (Gpm_Event *event, WInput *in)
 } 
 
 WInput *
-input_new (int y, int x, int color, int len, char *def_text, char *tkname)
+input_new (int y, int x, int color, int len, const char *def_text, char *tkname)
 {
     WInput *in = g_new (WInput, 1);
     int initial_buffer_len;
